// Constructor for a item that allows movement in 8 directions + up/down the structure
// You can move north, east, south, west, northwest, northeast, southeast and southwest
// item is an svg element 
// ax, ay, dx, dy are arrays for navItems, they will be sorted to allow nav to move in the direction
function eightWayNavItem(item, svg, centerCardinal, centerDiagonal) {
	this.cardinal = new navItem(item, false, this, svg, centerCardinal);
	this.diagonal = new navItem(item, true, this, svg, centerDiagonal);
}

// Moves in directions around a compass
eightWayNavItem.prototype.east = function() {
	return this.cardinal.greaterX().nav8item;
}

eightWayNavItem.prototype.north = function() {
	return this.cardinal.lesserY().nav8item;
}

eightWayNavItem.prototype.south = function() {
	return this.cardinal.greaterY().nav8item;
}

eightWayNavItem.prototype.west = function() {
	return this.cardinal.lesserX().nav8item;
}

eightWayNavItem.prototype.northeast = function() {
	return this.diagonal.greaterX().nav8item;
}

eightWayNavItem.prototype.northwest = function() {
	return this.diagonal.lesserY().nav8item;
}

eightWayNavItem.prototype.southeast = function() {
	return this.diagonal.greaterY().nav8item;
}

eightWayNavItem.prototype.southwest = function() {
	return this.diagonal.lesserX().nav8item;
}

// Non directional moves 
// Move in feature order (next, previous)
eightWayNavItem.prototype.nextFeature = function(forward) {
	return (forward) ? this.navFeature.nextFeature(this) : this.navFeature.previousFeature(this);
}

// Move into this object
eightWayNavItem.prototype.down = function() {
	// Most of the time this will do nothing, only useful when this object is explorable
	return (this.childWithRegard) ? this.childWithRegard.getPointOfRegard() : this;
}

// Move out of this object to parent object
eightWayNavItem.prototype.up = function() {
	return this.navFeature.up(this);
}

//Sets the navFeature to move down to. 
eightWayNavItem.prototype.setChildWithRegard = function(child) {
	this.childWithRegard = child;
}

// Sets the sort arrays fo the navItems and the navFeature
eightWayNavItem.prototype.setArrays = function(x, y, dx, dy, navFeature) {
	this.cardinal.setArrays(x, y);
	this.diagonal.setArrays(dx, dy);
	this.navFeature = navFeature;
}

eightWayNavItem.prototype.isItem = function(item) {
	return this.cardinal.item == item;
}

eightWayNavItem.prototype.getItem = function() {
	return this.cardinal.item;
}

//Container for navigation feature, remembers point of regard 
function navFeature(parent) {
	this.parent = parent;
}

// Set the next feature 
navFeature.prototype.setNext = function(next) {
	this.next = next;
	next.previous = this;
}

// Returns the point of regard in the next feature.
navFeature.prototype.nextFeature = function(child) {
	this.pointOfRegard = child;
	if (this.next) {
		return this.next.makePointOfRegard();
	}
	return this.pointOfRegard;
}

// Returns the point of regard in the previous feature.
navFeature.prototype.previousFeature = function(child) {
	this.pointOfRegard = child;
	if (this.previous) {
		return this.previous.makePointOfRegard();
	}
	return this.pointOfRegard;
}

// Returns the point of regard in this feature and notifies the parent this is the active feature
navFeature.prototype.makePointOfRegard = function() {
	if (this.parent) {
		this.parent.setChildWithRegard(this);
	}
	return this.pointOfRegard;
}

// Returns the point of regard of this feature.
navFeature.prototype.getPointOfRegard = function() {
	return this.pointOfRegard;
}

// Moves to the parent navigation item
navFeature.prototype.up = function(child) {
	this.pointOfRegard = child;
	if (this.parent) {
		this.parent.setChildWithRegard(this);
		return this.parent;
	}
	return child;
}

